/*
 * Copyright (c) Infinitegra Inc, All rights reserved.
 */

package infinitegra.owlift.sample.owliftsampleapp;

import android.app.Dialog;
import android.os.Bundle;
import android.view.View;

/**
 * OwliftDialogCallbacks
 * 
 * dialog設定で呼び出すcallbackのインターフェイス
 */
public interface OwliftDialogCallbacks
{
	/**
	 * ダイアログのボタン及びリストを押下した際のイベント処理.
	 * 
	 * @param tag
	 *        Fragmentのタグ
	 * @param args
	 *        setParamsで渡されたパラメータ
	 * @param which
	 *        DialogInterfaceのID
	 */
	void onDialogClicked(String tag, Bundle args, int which);

	/**
	 * ダイアログがキャンセルされた際のイベント処理.
	 * 
	 * @param tag
	 *        Fragmentのタグ
	 * @param args
	 *        setParamsで渡されたパラメータ
	 */
	void onDialogCancelled(String tag, Bundle args);

	/**
	 * alert dialogのViewを設定した際に呼び出される
	 * 
	 * @param view
	 *        view
	 * @param viewId
	 *        viewのlayoutのIDを指定
	 */
	void onCreateDialogView(View view, int viewId);

	/**
	 * dialogのonActivityCreated
	 */
	void onActivityCreated(View view, int viewId);

	/**
	 * alert dialogのonStart時に呼び出される
	 * 
	 * @param dialog
	 *        onCreateで生成されたdialog
	 * @param args
	 *        setParamsで渡されたパラメータ
	 */
	void onStart(Dialog dialog, Bundle args);

	/**
	 * neutralのボタンが押された時のイベント処理
	 * 
	 * @param tag
	 *        Fragmentのタグ
	 * @param args
	 *        setParamsで渡されたパラメータ
	 * @param which
	 *        DialogInterfaceのID
	 */
	void onDialogNeutralClicked(String tag, Bundle args, int which);
}
