/*
 * Copyright (c) Infinitegra Inc, All rights reserved.
 */

package infinitegra.owlift.sample.owliftsampleapp;

/*
 * OwliftDialogFragment
 *
 * ダイアログクラス
 * OKやCancelのCallbackを呼び出す
 */

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.DialogFragment;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;

public class OwliftDialogFragment extends DialogFragment
{
	private static final String ARG_PARAMS = "params";
	public static final String ARG_VIEW_ID = "view_id";
	public static final String ARG_TITLE = "title";
	public static final String ARG_TITLE_STRING = "titlestr";
	public static final String ARG_POSITIVE_BUTTON = "positiveButton";
	public static final String ARG_NEGATIVE_BUTTON = "negativeButton";
	public static final String ARG_NEUTRAL_BUTTON = "neutralButton";
	public static final String ARG_MESSAGE = "message";
	public static final String ARG_POSITIVE_BUTTON_ID = "positiveButtonId";
	public static final String ARG_NEGATIVE_BUTTON_ID = "negativeButtonId";
	public static final String ARG_NEUTRAL_BUTTON_ID = "neutralButtonId";

	private OwliftDialogCallbacks callbacks;
	private int viewId = 0;

	@Override
	public void onActivityCreated(Bundle savedInstanceState)
	{
		super.onActivityCreated(savedInstanceState);
		if (viewId != 0) {
			LayoutInflater inflater = LayoutInflater.from(getActivity());
			View inputView = inflater.inflate(viewId, null);
			callbacks.onActivityCreated(inputView, viewId);
		}
	}

	@Override
	public Dialog onCreateDialog(Bundle savedInstanceState)
	{
		if (getTargetFragment() != null
			&& getTargetFragment() instanceof OwliftDialogCallbacks) {
			callbacks = (OwliftDialogCallbacks)getTargetFragment();
		} else if (getActivity() instanceof OwliftDialogCallbacks) {
			callbacks = (OwliftDialogCallbacks)getActivity();
		} else {
			callbacks = new OwliftDialogCallbacks() {
				@Override
				public void onDialogClicked(String tag, Bundle args, int which)
				{
					// do nothing.
				}

				@Override
				public void onDialogCancelled(String tag, Bundle args)
				{
					// do nothing.
				}

				@Override
				public void onCreateDialogView(View view, int viewId)
				{
					// nop.
				}

				@Override
				public void onStart(Dialog dialog, Bundle args)
				{
					// nop.
				}

				@Override
				public void onDialogNeutralClicked(String tag, Bundle args,
					int which)
				{
					// nop
				}

				@Override
				public void onActivityCreated(View view, int viewId)
				{
					// nop.
				}
			};
		}

		DialogInterface.OnClickListener negativeListener =
			new DialogInterface.OnClickListener() {
				@Override
				public void onClick(DialogInterface dialog, int which)
				{
					onCancel(dialog);
				}
			};
		DialogInterface.OnClickListener positiveListener =
			new DialogInterface.OnClickListener() {
				@Override
				public void onClick(DialogInterface dialog, int which)
				{
					callbacks.onDialogClicked(getTag(), getArguments()
						.getBundle(ARG_PARAMS), which);
				}
			};
		DialogInterface.OnClickListener neutralListener =
			new DialogInterface.OnClickListener() {
				@Override
				public void onClick(DialogInterface dialog, int which)
				{
					callbacks.onDialogNeutralClicked(getTag(), getArguments()
						.getBundle(ARG_PARAMS), which);
				}
			};

		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		Bundle args = getArguments();

		if (args.containsKey(ARG_VIEW_ID)) {
			LayoutInflater inflater = LayoutInflater.from(getActivity());
			viewId = args.getInt(ARG_VIEW_ID);
			View inputView = inflater.inflate(viewId, null);
			builder.setView(inputView);
			callbacks.onCreateDialogView(inputView, viewId);
		}
		if (args.containsKey(ARG_TITLE)) {
			builder.setTitle(args.getInt(ARG_TITLE));
		} else if (args.containsKey(ARG_TITLE_STRING)) {
			builder.setTitle(args.getString(ARG_TITLE_STRING));
		}

		if (args.getBoolean(ARG_NEGATIVE_BUTTON)) {
			String negativeBtnName = getButtonName(ARG_NEGATIVE_BUTTON_ID);
			if (negativeBtnName == null || negativeBtnName.length() <= 0) {
				negativeBtnName = "Cancel";
			}
			builder.setNegativeButton(negativeBtnName, negativeListener);
		}
		if (args.getBoolean(ARG_POSITIVE_BUTTON)) {
			String positiveBtnName = getButtonName(ARG_POSITIVE_BUTTON_ID);
			if (positiveBtnName == null || positiveBtnName.length() <= 0) {
				positiveBtnName = "OK";
			}
			builder.setPositiveButton(positiveBtnName, positiveListener);
		}
		if (args.getBoolean(ARG_NEUTRAL_BUTTON)) {
			String neutralBtnName = getButtonName(ARG_NEUTRAL_BUTTON_ID);
			if (neutralBtnName == null || neutralBtnName.length() <= 0) {
				neutralBtnName = "close";
			}
			builder.setNeutralButton(neutralBtnName, neutralListener);
		}
		if (args.containsKey(ARG_MESSAGE)) {
			builder.setMessage(args.getInt(ARG_MESSAGE));
		}

		return builder.create();
	}

	private String getButtonName(String buttonId)
	{
		Bundle args = getArguments();
		if (args.containsKey(buttonId)) {
			int id = args.getInt(buttonId);
			String name = getString(id);
			if (name != null && name.length() > 0) {
				return name;
			}
		}
		return null;
	}

	public void onClickOK(int which)
	{
		callbacks.onDialogClicked(getTag(), getArguments(), which);
		dismiss();
	}

	@Override
	public void onCancel(DialogInterface dialog)
	{
		callbacks.onDialogCancelled(getTag(),
			getArguments().getBundle(ARG_PARAMS));
		super.onCancel(dialog);
	}

	@Override
	public void onStart()
	{
		super.onStart();
		Bundle args = getArguments();
		callbacks.onStart(getDialog(), args);
	}

	public static class Builder
	{
		private int viewId = 0;
		private Bundle params = null;
		private int titleId = 0;
		private String title = null;
		private boolean positiveBtn = true;
		private boolean negativeBtn = true;
		private boolean neutralBtn = false;
		private int messageId = 0;
		private int positiveBtnId = 0;
		private int negativeBtnId = 0;
		private int neutralBtnId = 0;

		public void clear()
		{
			viewId = 0;
			params = null;
			titleId = 0;
			title = null;
			messageId = 0;
			positiveBtn = true;
			negativeBtn = true;
			neutralBtn = false;
			positiveBtnId = 0;
			negativeBtnId = 0;
			neutralBtnId = 0;
		}

		public void setViewId(int displayViewId)
		{
			viewId = displayViewId;
		}

		public void setBundle(Bundle bundle)
		{
			params = bundle;
		}

		public void setTitle(int title)
		{
			titleId = title;
		}

		public void setTitle(String title)
		{
			this.title = title;
		}

		public void setPositiveButton(boolean positiveButton)
		{
			positiveBtn = positiveButton;
		}

		public void setNegativeButton(boolean negativeButton)
		{
			negativeBtn = negativeButton;
		}

		public void setNeutralButton(boolean neutralButton)
		{
			neutralBtn = neutralButton;
		}

		public void setMessage(int message)
		{
			messageId = message;
		}

		public void setPositiveButtonId(int id)
		{
			positiveBtnId = id;
		}

		public void setNegativeButtonId(int id)
		{
			negativeBtnId = id;
		}

		public void setNeutralButtonId(int id)
		{
			neutralBtnId = id;
		}

		public OwliftDialogFragment create()
		{
			OwliftDialogFragment f = new OwliftDialogFragment();

			Bundle args = new Bundle();
			if (params == null) {
				params = new Bundle();
			}

			if (viewId != 0) {
				args.putInt(ARG_VIEW_ID, viewId);
				params.putInt(ARG_VIEW_ID, viewId);
			}
			if (titleId != 0) {
				args.putInt(ARG_TITLE, titleId);
			} else if (title != null) {
				args.putString(ARG_TITLE_STRING, title);
			}
			if (messageId != 0) {
				args.putInt(ARG_MESSAGE, messageId);
			}
			if (positiveBtnId != 0) {
				args.putInt(ARG_POSITIVE_BUTTON_ID, positiveBtnId);
			}
			if (negativeBtnId != 0) {
				args.putInt(ARG_NEGATIVE_BUTTON_ID, negativeBtnId);
			}
			if (neutralBtnId != 0) {
				args.putInt(ARG_NEUTRAL_BUTTON_ID, neutralBtnId);
			}

			args.putBoolean(ARG_POSITIVE_BUTTON, positiveBtn);
			args.putBoolean(ARG_NEGATIVE_BUTTON, negativeBtn);
			args.putBoolean(ARG_NEUTRAL_BUTTON, neutralBtn);

			args.putBundle(ARG_PARAMS, params);
			f.setArguments(args);
			return f;
		}
	}
}
