/*
 * Copyright (c) Infinitegra Inc, All rights reserved.
 */

package infinitegra.owlift.sample.owliftsampleapp;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.RectF;

import java.nio.ByteBuffer;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import infinitegra.owlift.DecodeFunc;
import infinitegra.owlift.OwliftErrorId;
import infinitegra.owlift.OwliftTempTable;
import infinitegra.owlift.UsbOwliftDev;
import infinitegra.owlift.OwliftUtil;

public class OwliftDisp
{
	private static final int OWLIFT_RAW_WIDTH = 80;
	private static final int OWLIFT_DISP_WIDTH = 80;
	private static final int OWLIFT_RAW_HEIGHT = 64;
	private static final int OWLIFT_DISP_HEIGHT = 60;
	private static final int OWLIFT_L3_RAW_WIDTH = 160;
	private static final int OWLIFT_L3_DISP_WIDTH = 160;
	private static final int OWLIFT_L3_RAW_HEIGHT = 122;
	private static final int OWLIFT_L3_DISP_HEIGHT = 120;

	public static final int OWLIFT_VID = 0x2588;
	public static final int OWLIFT_PID = 0x2300;
	public static final int OWLIFT_PID_B = 0x2301;
	public static final int OWLIFT_PID_E = 0x2302;
	public static final int OWLIFT_PID_D = 0x2320;
	public static final int OWLIFT_PID_F = 0x2303;

	public static final int MARKER_TYPE_HIGH = 0;
	public static final int MARKER_TYPE_LOW = 1;
	public static final int MARKER_TYPE_POINT = 2;

	public static final int TEMP_UNIT_CELSIUS = 0;
	public static final int TEMP_UNIT_FAHLENHEIT = 1;
	public static final int TEMP_UNIT_KELVIN = 2;

	public static final int OWLIFT_TEMP_BG_RECT_SIZE = 50;
	public static final int OWLIFT_DISPLAY_DATE_MARGIN = 5;

	private UsbOwliftDev dev = null;

	// owlift marker
	private boolean isMarkerHigh = false;
	private boolean isMarkerLow = false;
	private boolean isMarker1 = false;
	private Canvas canvas = null;
	private Bitmap bitmap = null;
	private Paint txtPaint;
	private Paint bgPaint;
	private Paint datePaint;
	private Paint dateBgPaint;
	private int dateTxtHt;

	private boolean isDispDate;
	private RectF dateBgRect;

	private Paint highPaint, lowPaint, marker1Paint;

	private byte[] magBuf = null;
	private int tempUnit = TEMP_UNIT_CELSIUS;
	private int marker1x, marker1y;

	private OwliftTempTable tempTable;
	private double emissivityR4 =
			OwliftTempTable.getEmissivity(MainActivity.OWLIFT_DEFAULT_EMISSIVITY);
	private double emissivity = MainActivity.OWLIFT_DEFAULT_EMISSIVITY;

	OwliftDisp()
	{
		txtPaint = new Paint();
		txtPaint.setAntiAlias(true);
		txtPaint.setColor(Color.BLACK);
		txtPaint.setTextSize(15f);

		int bgColor = Color.argb(0x80, 0xF0, 0xF0, 0xF0);

		bgPaint = new Paint();
		bgPaint.setColor(bgColor);
		bgPaint.setAntiAlias(true);

		highPaint = new Paint();
		highPaint.setColor(Color.argb(0xFF, 0xFF, 0x62, 0x32));

		marker1Paint = new Paint();
		marker1Paint.setColor(Color.WHITE);

		lowPaint = new Paint();
		lowPaint.setColor(Color.argb(0xFF, 0xA7, 0x6B, 0xDF));

		datePaint = new Paint();
		datePaint.setColor(Color.BLACK);
		datePaint.setTextSize(12f);
		datePaint.setAntiAlias(true);
		dateBgPaint = new Paint();
		dateBgPaint.setColor(bgColor);
		dateBgPaint.setAntiAlias(true);

		dateTxtHt = (int)(datePaint.getFontMetrics().descent -
			datePaint.getFontMetrics().ascent);
		int dateTxtHeightSize = (int)(datePaint.getFontMetrics().bottom -
			datePaint.getFontMetrics().top);
		String sample = "MM/dd yyyy";
		float[] widths = new float[sample.length()];
		int strWdSz = 0;

		int count = datePaint.getTextWidths(sample, widths);
		for (int i = 0; i < count; i++) {
			strWdSz += widths[i];
		}
		int margin = OWLIFT_DISPLAY_DATE_MARGIN;
		dateBgRect = new RectF(margin, margin,
			strWdSz + margin + 2, dateTxtHeightSize * 2 + margin + 2);

	}

	boolean getTempMarker(int type)
	{
		boolean ret = false;
		switch (type) {
		case MARKER_TYPE_HIGH:
			ret = isMarkerHigh;
			break;
		case MARKER_TYPE_LOW:
			ret = isMarkerLow;
			break;
		case MARKER_TYPE_POINT:
			ret = isMarker1;
			break;
		}
		return ret;
	}

	void setTempMarkerEnable(int type, boolean enable)
	{
		switch (type) {
		case MARKER_TYPE_HIGH:
			isMarkerHigh = enable;
			break;
		case MARKER_TYPE_LOW:
			isMarkerLow = enable;
			break;
		case MARKER_TYPE_POINT:
			isMarker1 = enable;
			break;
		}
	}

	boolean isDispDate()
	{
		return isDispDate;
	}

	void setDispDate(boolean enable)
	{
		isDispDate = enable;
	}

	void setEmissivity(double emissivity)
	{
		this.emissivity = emissivity;
		emissivityR4 = OwliftTempTable.getEmissivity(emissivity);
	}

	double getEmissivity()
	{
		return emissivity;
	}

	byte[] getDispBuf(DecodeFunc decode, byte[] buf, int width,
		int height, int texWX, int texWY)
	{
		byte[] ret;
		// 3倍に拡大。および温度データ書き込み
		if (magBuf == null) {
			magBuf = new byte[texWX * texWY * 2 *
				MainActivity.OWLIFT_MAGNIFY_SIZE];
		}

		// sdwとddwは同じ。
		int err = OwliftUtil.magnifyBuf3(width, height,
			texWX, texWX, buf, magBuf);
		if (err != OwliftErrorId.ERR_OK) {
			return null;
		}

		ret = magBuf;
		Canvas drawCanvas = null;
		if (isMarkerHigh || isMarkerLow || isMarker1 || isDispDate) {
			drawCanvas = getDrawCanvas(ret, texWX, texWY);
		}
		// 温度データを取得しておく
		OwliftTempTable table = getTempTable(decode, width, height);
		if (table != null) {
			drawTempMarker(table, width, height, drawCanvas);
		}

		if (isDispDate) {
			drawDate(drawCanvas);
		}
		if (drawCanvas != null) {
			byte[] ary = getDrawArray(texWX, texWY);
			if (ary != null) {
				ret = ary;
			}
		}
		return ret;
	}

	private void drawTempMarker(OwliftTempTable tempTable, int width,
		int height,
		Canvas drawCanvas)
	{
		if (tempTable == null || (!isMarkerHigh && !isMarkerLow && !isMarker1)
			||
			drawCanvas == null) {
			return;
		}
		double high = 0, low = 1000;
		int hw = -1, hh = -1, lw = -1, lh = -1;

		if (isMarkerHigh || isMarkerLow) {
			double temp;
			for (int h = 0; h < height; h++) {
				for (int w = 0; w < width; w++) {
					temp = tempTable.getTemp(w, h, emissivityR4);
					if (isMarkerHigh && high < temp) {
						hw = w;
						hh = h;
						high = temp;
					}
					if (isMarkerLow && low > temp) {
						lw = w;
						lh = h;
						low = temp;
					}
				}
			}
		}
		if (isMarkerHigh) {
			drawMarker(drawCanvas, width, height,
				MainActivity.OWLIFT_MAGNIFY_SIZE, hw, hh,
				high, highPaint);
		}
		if (isMarkerLow) {
			drawMarker(drawCanvas, width, height,
				MainActivity.OWLIFT_MAGNIFY_SIZE, lw, lh,
				low, lowPaint);
		}
		if (isMarker1) {
			double temp = tempTable.getTemp(marker1x, marker1y, emissivityR4);
			drawMarker(drawCanvas, width, height, MainActivity.OWLIFT_MAGNIFY_SIZE,
					marker1x, marker1y,
					temp, marker1Paint);
		}

	}

	private byte[] getDrawArray(int texWX, int texWY)
	{
		if (bitmap == null) {
			return null;
		}
		ByteBuffer dstBuf = ByteBuffer.allocate(texWX * texWY * 2);
		bitmap.copyPixelsToBuffer(dstBuf);
		return dstBuf.array();
	}

	private Canvas getDrawCanvas(byte[] buf, int texWX, int texWY)
	{
		// canvasを使いまわすためにメンバに持っている
		if (canvas == null) {
			bitmap = Bitmap.createBitmap(texWX, texWY, Bitmap.Config.RGB_565);
			canvas = new Canvas(bitmap);
		}
		ByteBuffer byteBuf = ByteBuffer.wrap(buf);
		bitmap.copyPixelsFromBuffer(byteBuf);
		canvas.drawBitmap(bitmap, new Rect(0, 0, texWX, texWY),
			new Rect(0, 0, texWX, texWY), null);
		return canvas;
	}

	private void drawDate(Canvas drawCanvas)
	{
		if (drawCanvas == null) {
			return;
		}

		Date d = new Date();
		SimpleDateFormat sdf = new SimpleDateFormat("MM/dd/yyyy",
			Locale.getDefault());
		SimpleDateFormat sdf2 = new SimpleDateFormat("HH:mm:ss",
			Locale.getDefault());

		drawCanvas.drawRoundRect(dateBgRect, 8, 8, dateBgPaint);
		drawCanvas.drawText(sdf.format(d), OWLIFT_DISPLAY_DATE_MARGIN + 2,
			dateTxtHt + OWLIFT_DISPLAY_DATE_MARGIN, datePaint);
		drawCanvas.drawText(sdf2.format(d), OWLIFT_DISPLAY_DATE_MARGIN + 2,
			dateTxtHt * 2 + OWLIFT_DISPLAY_DATE_MARGIN, datePaint);
	}

	private void drawMarker(Canvas drawCanvas, int width, int height,
		int magnify, int x, int y, double temp, Paint pointPaint)
	{
		if (x < 0 || y < 0) {
			return;
		}
		int tempResolution = dev.getTempResolution();
		String vstr = null;
		if (tempResolution == 100 && temp > 655.15) {
			switch (tempUnit) {
			case TEMP_UNIT_CELSIUS:
				vstr = ">382C";
				break;
			case TEMP_UNIT_FAHLENHEIT:
				vstr = ">720F";
				break;
			case TEMP_UNIT_KELVIN:
				vstr = ">655K";
				break;
			}
		} else if (tempResolution == 10 && temp > 710.1) {
			switch (tempUnit) {
			case TEMP_UNIT_CELSIUS:
				vstr = ">437C";
				break;
			case TEMP_UNIT_FAHLENHEIT:
				vstr = ">818F";
				break;
			case TEMP_UNIT_KELVIN:
				vstr = ">710K";
				break;
			}
		} else if (temp > 0) {
			switch (tempUnit) {
			case TEMP_UNIT_CELSIUS:
				vstr = String.format(Locale.ENGLISH, "%3.0f", (temp - 273.15 + 0.5)) + "°C";
				break;
			case TEMP_UNIT_FAHLENHEIT:
				vstr =
					(String.format(Locale.ENGLISH, "%3.0f", (temp - 273.15) * 1.8 + 32.0 + 0.5))
						+ "F";
				break;
			case TEMP_UNIT_KELVIN:
				vstr = String.format(Locale.ENGLISH, "%3.0f", (temp + 0.5)) + "K";
				break;
			}
		}
		if (vstr == null) {
			return;
		}
		int wx = x * magnify;
		int wy = y * magnify;

		int bgl = wx - OWLIFT_TEMP_BG_RECT_SIZE / 2;
		int bgr = wx + OWLIFT_TEMP_BG_RECT_SIZE / 2;
		int bgt = wy - OWLIFT_TEMP_BG_RECT_SIZE / 2;
		int bgb = wy + OWLIFT_TEMP_BG_RECT_SIZE / 2;
		int wd = width * magnify;
		int ht = height * magnify;
		if (bgl < 0) {
			bgr -= bgl;
			bgl = 0;
		} else if (wd < bgr) {
			bgl -= bgr - wd;
			bgr = wd;
		}
		if (bgt < 0) {
			bgb -= bgt;
			bgt = 0;
		} else if (ht < bgb) {
			bgt -= bgb - ht;
			bgb = ht;
		}
		RectF bgRect = new RectF(bgl, bgt, bgr, bgb);
		drawCanvas.drawRoundRect(bgRect, 10, 10, bgPaint);

		float[] widths = new float[vstr.length()];
		int strWdSz = 0;

		txtPaint.getTextWidths(vstr, widths);
		for (int i = 0; i < vstr.length(); i++) {
			strWdSz += widths[i] * 100;
		}
		strWdSz = strWdSz / 100;

		int txtW = wx - strWdSz / 2;
		int strHtSz = (int)(txtPaint.getFontMetrics().descent -
			txtPaint.getFontMetrics().ascent);
		int txtH = wy - 4;
		if (wx + strWdSz > wd) {
			txtW = wd - strWdSz;
		} else if (txtW < 0) {
			txtW = 0;
		}
		if (txtH - strHtSz < 0) {
			txtH = wy + strHtSz;
		}
		drawCanvas.drawText(vstr, txtW, txtH, txtPaint);
		int pl = wx - 2 <= 0 ? 2 : wx + 2 > wd ? wx - 4 : wx - 2;
		int pt = wy - 2 <= 0 ? 2 : wy + 2 > ht ? ht - 4 : wy - 2;
		int pr = pl + 4;
		int pb = pt + 4;
		Rect r = new Rect(pl, pt, pr, pb);
		drawCanvas.drawRect(r, pointPaint);

	}

	int getTempUnit()
	{
		return tempUnit;
	}

	void setTempUnit(int tempUnit)
	{
		this.tempUnit = tempUnit;
	}

	private OwliftTempTable getTempTable(DecodeFunc decode, int width,
		int height)
	{
		if (decode == null) {
			return null;
		}
		if (tempTable == null) {
			tempTable = new OwliftTempTable(width, height);
			tempTable.setOwliftDev(dev);
		}
		int ret = decode.getTempTable(tempTable);
		if (ret == OwliftErrorId.ERR_OK) {
			return tempTable;
		}
		return null;
	}

	void setTempMarker(int x, int y)
	{
		marker1x = x;
		marker1y = y;
	}

	void setUsbOwliftDev(UsbOwliftDev dev)
	{
		this.dev = dev;
		tempTable = null;
	}

	int getOwliftRawWidth()
	{

		if (dev != null && dev.getPID() == OWLIFT_PID_F) {
			return OWLIFT_L3_RAW_WIDTH;
		}
		return OWLIFT_RAW_WIDTH;
	}
	int getOwliftRawHeight()
	{
		if (dev != null && dev.getPID() == OWLIFT_PID_F) {
			return OWLIFT_L3_RAW_HEIGHT;
		}
		return OWLIFT_RAW_HEIGHT;
	}
	int getOwliftDispWidth()
	{
		if (dev != null && dev.getPID() == OWLIFT_PID_F) {
			return OWLIFT_L3_DISP_WIDTH;
		}
		return OWLIFT_DISP_WIDTH;
	}
	int getOwliftDispHeight()
	{
		if (dev != null && dev.getPID() == OWLIFT_PID_F) {
			return OWLIFT_L3_DISP_HEIGHT;
		}
		return OWLIFT_DISP_HEIGHT;
	}
}
